% Concepts and Methods of 2D Infrared Spectroscopy
% Peter Hamm and Martin T. Zanni
% This matlab program duplicates Figs. 4.14 and 10.8

% the number of excited states, their frequencies, and dipoles, all
% taken from Gaussian output. An offset frequency is subtracted
% (and added back after the Fourier transform) to save computation
% time. The two transition dipoles are in the x- and y-directions,
% respectively. 

n = 2;
w = [1889.184 1947.621];
mu = [sqrt(0.908) 0 0 ; 0 sqrt(0.671) 0];
w_off = 1915;
w = w - w_off;

% The number of doubly excited states, their frequencies, and their
% transition dipoles taken from the Gaussian output. We use the
% harmonic approximation for the transition dipoles.

n2 = 3;
w2 = [3767.517 3883.911 3813.698];
w2 = w2 - 2*w_off;
mu2 = zeros(n2,3,n);
mu2(:,:,1) = [sqrt(2).*mu(1,:) ; 0,0,0 ; mu(2,:)];
mu2(:,:,2) = [ 0,0,0 ; sqrt(2).*mu(2,:) ; mu(1,:)];


% Parameters of the simulation
n_t = 128;
n_zp = 2*n_t; % the zeropadded length
dt = 0.25;
T2 = 2; %dephasing time
c = 0.188; %unit conversion factor
t2 = 0; %the population time

% Calculate response functions for the rephasing and non-rephasing
% diagrams (Eqs. 10.11, 10.12, and 10.13). The first time points
% need to be halved (Sec. 9.5.3).

R_r = zeros(n_t,n_t);
R_nr = zeros(n_t,n_t);
t=0:dt:(n_t-1)*dt;
[T1,T3] = meshgrid(t,t);

% first calculate all rephasing diagrams
for j = 1:n
  for i  = 1:n
    
    mui = sqrt(mu(i,:)*mu(i,:)');
    muj = sqrt(mu(j,:)*mu(j,:)');
    cos1 = mu(i,:)*mu(j,:)'/(mui*muj);
    angle = (1 + 2*cos1^2 )/15;
    dipole = mui^2*muj^2;
    
    % rephasing diagram R1
    R_r = R_r - dipole*angle*exp(+ 1i*w(j)*c.*T1 ...
				 - 1i*w(i)*c.*T3 ...
				 + 1i*(w(j)-w(i))*c*t2 ...
				 - (T1+T3)/T2);
    % rephasing diagram R2
    R_r = R_r - dipole*angle*exp(+ 1i*w(j)*c.*T1 ...
				 - 1i*w(i)*c.*T3 ...
				 - (T1+T3)/T2);
    
    for k = 1:n2
      mujk = sqrt(mu2(k,:,j)*mu2(k,:,j)');
      muik = sqrt(mu2(k,:,i)*mu2(k,:,i)');
      
      dipole = mui*muj*muik*mujk;
      if muik~=0 &&  mujk~=0
	cos2 = mu2(k,:,i)*mu2(k,:,j)'/(muik*mujk);
      else
	cos2 = 0;
      end
      if muik~=0 
	cos3 = mu(i,:)*mu2(k,:,i)'/(mui*muik);
      else
	cos3 = 0;
      end
      if mujk~=0
	cos4 = mu(j,:)*mu2(k,:,j)'/(muj*mujk);
      else
	cos4 = 0;
      end
      if mujk~=0
	cos5 = mu(i,:)*mu2(k,:,j)'/(mui*mujk);
      else
	cos5 = 0;
      end
      if muik~=0
	cos6 = mu(j,:)*mu2(k,:,i)'/(muj*muik);
      else
	cos6 = 0;
      end
      angle = (cos1*cos2+cos3*cos4+cos5*cos6)/15;
      
      %rephasing diagram R3
      R_r = R_r + dipole*angle*exp(+ 1i*w(j)*c.*T1 ...
				   - 1i*(w2(k)-w(j))*c.*T3 ...
				   + 1i*(w(j)-w(i))*c.*t2 ...
				   - (T1+T3)/T2);
    end
  end
end

% now non-rephasing diagrams
for j = 1:n
  for i  = 1:n
    mui = sqrt(mu(i,:)*mu(i,:)');
    muj = sqrt(mu(j,:)*mu(j,:)');
    cos1 = mu(i,:)*mu(j,:)'/(mui*muj);
    angle = (1 + 2*cos1^2 )/15;
    dipole = mui^2*muj^2;
      
    % non-rephasing diagram R4
    R_nr = R_nr - dipole*angle*exp(- 1i*w(j)*c.*T1 ...
				   - 1i*w(j)*c.*T3 ... %?
				   - 1i*(w(j)-w(i))*c*t2 ...
				   - (T1+T3)/T2);
    % non-rephasing diagram R5
    R_nr = R_nr - dipole*angle*exp(- 1i*w(j)*c.*T1 ...
				   - 1i*w(i)*c.*T3 ...
				   - (T1+T3)/T2);
    
    for k = 1:n2
      mujk = sqrt(mu2(k,:,j)*mu2(k,:,j)');
      muik = sqrt(mu2(k,:,i)*mu2(k,:,i)');
      
      dipole = mui*muj*muik*mujk;
      if muik~=0 && mujk~=0
	cos2 = mu2(k,:,i)*mu2(k,:,j)'/(muik*mujk);
      else
	cos2 = 0;
      end
      if muik~=0 
	cos3 = mu(i,:)*mu2(k,:,i)'/(mui*muik);
      else
	cos3 = 0;
      end
      if mujk~=0
	cos4 = mu(j,:)*mu2(k,:,j)'/(muj*mujk);
      else
	cos4 = 0;
      end
      if mujk~=0
	cos5 = mu(i,:)*mu2(k,:,j)'/(mui*mujk);
      else
	cos5 = 0;
      end
      if muik~=0
	cos6 = mu(j,:)*mu2(k,:,i)'/(muj*muik);
      else
	cos6 = 0;
      end
      angle = (cos1*cos2+cos3*cos4+cos5*cos6)/15;
      
      %non-rephasing diagram R6
      R_nr = R_nr + dipole*angle*exp(- 1i*w(j)*c.*T1 ...
				     - 1i*(w2(k)-w(i))*c.*T3 ...
				     - 1i*(w(j)-w(i))*c.*t2 ...
				     - (T1+T3)/T2);
    end
  end
end

% divide first points (by row and column) by 2
R_r(:,1) = R_r(:,1)./2;
R_r(1,:) = R_r(1,:)./2;
R_nr(:,1) = R_nr(:,1)./2;
R_nr(1,:) = R_nr(1,:)./2;

%what we have so far in the time domain
figure(1),clf
subplot(1,2,1)
contourf(real(R_r'),10); 
axis equal tight
subplot(1,2,2)
contourf(real(R_nr'),10);
axis equal tight

% do the fft
R_r = ifft2(R_r,n_zp,n_zp); %given the frequency definitions used
                            %above, use the ifft to get the
                            %frequencies right (Mathematica has the
                            %opposite definition of the fft by default)
R_nr = ifft2(R_nr,n_zp,n_zp);

%now frequency domain
figure(2),clf
subplot(1,2,1)
contourf(fftshift(real(R_r')),20); %pump-probe axis convention
%contourf(fftshift(real(R_r)),20; % the (omega_1, omega_3) axis convention
axis equal tight
subplot(1,2,2)
contourf(fftshift(real(R_nr')),20)
%contourf(fftshift(real(R_nr)),20)
axis equal tight

% flip R_r (being careful to keep zero frequency as the first time
% point), add the response functions, take the real part, and
% finally reorganize so that the 0 frequency is in the center
R = fftshift(real(fliplr(circshift(R_r,[0 -1]))+R_nr));
    

figure(3),clf
n_contours = 40;
MAX = max(abs(R(:)));
level_list = linspace(-MAX,MAX,n_contours+2);
dl = level_list(2)-level_list(1);
cmin =level_list(1)-dl/2;
cmax =level_list(end);

contourf(R',level_list) %use R' to display the pump-probe axis convention
%contourf(R,level_list) %use R to display the (omega_1, omega_3) axis convention
caxis([cmin cmax]);			
axis equal tight
